#include <iostream>
#include <memory>

#include "model-odb.hxx"

#include <odb/sqlite/database.hxx>

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#else
#include <dlfcn.h>
#endif


int main(int argc, char* argv[]) {
	try{

		const std::string dbname = (argc > 1) ? argv[1] : "testdb.sqlite";

		// Load the sqlite dynamic library
		std::clog << "Loading sqlite odb library ... " << std::flush;
#ifdef _WIN32
		HMODULE lib = ::LoadLibrary("model-sqlite.dll");
		std::clog << (lib ? "SUCCESS" : "FAILED!") << std::endl;
#else
#ifdef __APPLE__
		if (!dlopen("libmodel-sqlite.dylib", RTLD_NOW)) {
#else
		if (!dlopen("libmodel-sqlite.so", RTLD_NOW)) {
#endif
			std::clog << " FAILED!" << "\nReason: " << dlerror() << std::endl;
		} else {
			std::clog << " SUCCESS!" << std::endl;
		}
#endif


		//Create a database object
		const std::auto_ptr<odb::database> db(new odb::sqlite::database(dbname, SQLITE_OPEN_READWRITE /*| SQLITE_OPEN_CREATE*/));


		//Start a transaction
		odb::transaction t(db->begin());

		//db->tracer(odb::stderr_tracer);

		std::clog << "Load a simple object ... " << std::endl;
		{
			odb::object_traits<SimpleObject>::pointer_type obj = db->load<SimpleObject>(1);
			std::clog << "   ID: " << obj->id() << std::endl;
		}

		std::clog << "Querying simple objects ... " << std::endl;
		{
			odb::result<SimpleObject> r = db->query<SimpleObject>(odb::query<SimpleObject>::id == 1);
			for (odb::result<SimpleObject>::iterator it = r.begin(); it != r.end(); ++it) {
				std::clog << "   ID: " << it.load()->id() << std::endl;
			}
		}


		std::clog << "Load a composite object ... " << std::endl;
		{
			odb::object_traits<CompositeObject>::pointer_type obj = db->load<CompositeObject>(CompositeKey(1, 1));
			std::clog << "   ID: " << obj->id().key1() << ", " << obj->id().key2() << std::endl;
		}

		std::clog << "Querying composite objects ... " << std::endl;
		{
			odb::result<CompositeObject> r = db->query<CompositeObject>(odb::query<CompositeObject>::id.key1 == 1);
			for (odb::result<CompositeObject>::iterator it = r.begin(); it != r.end(); ++it) {
				std::clog << "   ID: " << it.load()->id().key1() << ", " << it.load()->id().key2() << std::endl;
			}
		}

		t.commit();
		std::clog << "Finished with no errors!" << std::endl;


	}
	catch (std::exception &e) {
		std::cerr << "ERROR: " << e.what() << std::endl;
		return -1;
	}

	return 0;
}
